@@ -8,9 +8,12 @@ import (
 
 	"github.com/mjl-/mox/dkim"
 	"github.com/mjl-/mox/dns"
+	"github.com/mjl-/mox/mlog"
 	"github.com/mjl-/mox/spf"
 )
 
+var pkglog = mlog.New("dmarc", nil)
+
 func TestLookup(t *testing.T) {
 	resolver := dns.MockResolver{
 		TXT: map[string][]string{
@@ -21,15 +24,15 @@ func TestLookup(t *testing.T) {
 			"_dmarc.malformed.example.": {"v=DMARC1; p=none; bogus;"},
 			"_dmarc.example.com.":       {"v=DMARC1; p=none;"},
 		},
-		Fail: map[dns.Mockreq]struct{}{
-			{Type: "txt", Name: "_dmarc.temperror.example."}: {},
+		Fail: []string{
+			"txt _dmarc.temperror.example.",
 		},
 	}
 
 	test := func(d string, expStatus Status, expDomain string, expRecord *Record, expErr error) {
 		t.Helper()
 
-		status, dom, record, _, err := Lookup(context.Background(), resolver, dns.Domain{ASCII: d})
+		status, dom, record, _, _, err := Lookup(context.Background(), pkglog.Logger, resolver, dns.Domain{ASCII: d})
 		if (err == nil) != (expErr == nil) || err != nil && !errors.Is(err, expErr) {
 			t.Fatalf("got err %#v, expected %#v", err, expErr)
 		}
@@ -60,15 +63,15 @@ func TestLookupExternalReportsAccepted(t *testing.T) {
 			"example.com._report._dmarc.multiple.example.":  {"v=DMARC1; p=none;", "v=DMARC1"},
 			"example.com._report._dmarc.malformed.example.": {"v=DMARC1; p=none; bogus;"},
 		},
-		Fail: map[dns.Mockreq]struct{}{
-			{Type: "txt", Name: "example.com._report._dmarc.temperror.example."}: {},
+		Fail: []string{
+			"txt example.com._report._dmarc.temperror.example.",
 		},
 	}
 
 	test := func(dom, extdom string, expStatus Status, expAccepts bool, expErr error) {
 		t.Helper()
 
-		accepts, status, _, _, err := LookupExternalReportsAccepted(context.Background(), resolver, dns.Domain{ASCII: dom}, dns.Domain{ASCII: extdom})
+		accepts, status, _, _, _, err := LookupExternalReportsAccepted(context.Background(), pkglog.Logger, resolver, dns.Domain{ASCII: dom}, dns.Domain{ASCII: extdom})
 		if (err == nil) != (expErr == nil) || err != nil && !errors.Is(err, expErr) {
 			t.Fatalf("got err %#v, expected %#v", err, expErr)
 		}
@@ -84,7 +87,7 @@ func TestLookupExternalReportsAccepted(t *testing.T) {
 	test("example.com", "simple2.example", StatusNone, true, nil)
 	test("example.com", "one.example", StatusNone, true, nil)
 	test("example.com", "absent.example", StatusNone, false, ErrNoRecord)
-	test("example.com", "multiple.example", StatusNone, false, ErrMultipleRecords)
+	test("example.com", "multiple.example", StatusNone, true, nil)
 	test("example.com", "malformed.example", StatusPermerror, false, ErrSyntax)
 	test("example.com", "temperror.example", StatusTemperror, false, ErrDNS)
 }
@@ -100,8 +103,8 @@ func TestVerify(t *testing.T) {
 			"_dmarc.malformed.example.": {"v=DMARC1; p=none; bogus"},
 			"_dmarc.example.com.":       {"v=DMARC1; p=reject"},
 		},
-		Fail: map[dns.Mockreq]struct{}{
-			{Type: "txt", Name: "_dmarc.temperror.example."}: {},
+		Fail: []string{
+			"txt _dmarc.temperror.example.",
 		},
 	}
 
@@ -124,7 +127,7 @@ func TestVerify(t *testing.T) {
 		if err != nil {
 			t.Fatalf("parsing domain: %v", err)
 		}
-		useResult, result := Verify(context.Background(), resolver, from, dkimResults, spfResult, spfIdentity, true)
+		useResult, result := Verify(context.Background(), pkglog.Logger, resolver, from, dkimResults, spfResult, spfIdentity, true)
 		if useResult != expUseResult || !equalResult(result, expResult) {
 			t.Fatalf("verify: got useResult %v, result %#v, expected %v %#v", useResult, result, expUseResult, expResult)
 		}
@@ -137,7 +140,7 @@ func TestVerify(t *testing.T) {
 		[]dkim.Result{},
 		spf.StatusNone,
 		nil,
-		true, Result{true, StatusFail, dns.Domain{ASCII: "reject.example"}, &reject, nil},
+		true, Result{true, StatusFail, false, false, dns.Domain{ASCII: "reject.example"}, &reject, false, nil},
 	)
 
 	// Accept with spf pass.
@@ -145,7 +148,7 @@ func TestVerify(t *testing.T) {
 		[]dkim.Result{},
 		spf.StatusPass,
 		&dns.Domain{ASCII: "sub.reject.example"},
-		true, Result{false, StatusPass, dns.Domain{ASCII: "reject.example"}, &reject, nil},
+		true, Result{false, StatusPass, true, false, dns.Domain{ASCII: "reject.example"}, &reject, false, nil},
 	)
 
 	// Accept with dkim pass.
@@ -161,7 +164,7 @@ func TestVerify(t *testing.T) {
 		},
 		spf.StatusFail,
 		&dns.Domain{ASCII: "reject.example"},
-		true, Result{false, StatusPass, dns.Domain{ASCII: "reject.example"}, &reject, nil},
+		true, Result{false, StatusPass, false, true, dns.Domain{ASCII: "reject.example"}, &reject, false, nil},
 	)
 
 	// Reject due to spf and dkim "strict".
@@ -181,7 +184,7 @@ func TestVerify(t *testing.T) {
 		},
 		spf.StatusPass,
 		&dns.Domain{ASCII: "sub.strict.example"},
-		true, Result{true, StatusFail, dns.Domain{ASCII: "strict.example"}, &strict, nil},
+		true, Result{true, StatusFail, false, false, dns.Domain{ASCII: "strict.example"}, &strict, false, nil},
 	)
 
 	// No dmarc policy, nothing to say.
@@ -189,7 +192,7 @@ func TestVerify(t *testing.T) {
 		[]dkim.Result{},
 		spf.StatusNone,
 		nil,
-		false, Result{false, StatusNone, dns.Domain{ASCII: "absent.example"}, nil, ErrNoRecord},
+		false, Result{false, StatusNone, false, false, dns.Domain{ASCII: "absent.example"}, nil, false, ErrNoRecord},
 	)
 
 	// No dmarc policy, spf pass does nothing.
@@ -197,7 +200,7 @@ func TestVerify(t *testing.T) {
 		[]dkim.Result{},
 		spf.StatusPass,
 		&dns.Domain{ASCII: "absent.example"},
-		false, Result{false, StatusNone, dns.Domain{ASCII: "absent.example"}, nil, ErrNoRecord},
+		false, Result{false, StatusNone, false, false, dns.Domain{ASCII: "absent.example"}, nil, false, ErrNoRecord},
 	)
 
 	none := DefaultRecord
@@ -207,7 +210,7 @@ func TestVerify(t *testing.T) {
 		[]dkim.Result{},
 		spf.StatusPass,
 		&dns.Domain{ASCII: "none.example"},
-		true, Result{false, StatusPass, dns.Domain{ASCII: "none.example"}, &none, nil},
+		true, Result{false, StatusPass, true, false, dns.Domain{ASCII: "none.example"}, &none, false, nil},
 	)
 
 	// No actual reject due to pct=0.
@@ -218,7 +221,7 @@ func TestVerify(t *testing.T) {
 		[]dkim.Result{},
 		spf.StatusNone,
 		nil,
-		false, Result{true, StatusFail, dns.Domain{ASCII: "test.example"}, &testr, nil},
+		false, Result{true, StatusFail, false, false, dns.Domain{ASCII: "test.example"}, &testr, false, nil},
 	)
 
 	// No reject if subdomain has "none" policy.
@@ -229,7 +232,7 @@ func TestVerify(t *testing.T) {
 		[]dkim.Result{},
 		spf.StatusFail,
 		&dns.Domain{ASCII: "sub.subnone.example"},
-		true, Result{false, StatusFail, dns.Domain{ASCII: "subnone.example"}, &sub, nil},
+		true, Result{false, StatusFail, false, false, dns.Domain{ASCII: "subnone.example"}, &sub, false, nil},
 	)
 
 	// No reject if spf temperror and no other pass.
@@ -237,7 +240,7 @@ func TestVerify(t *testing.T) {
 		[]dkim.Result{},
 		spf.StatusTemperror,
 		&dns.Domain{ASCII: "mail.reject.example"},
-		true, Result{false, StatusTemperror, dns.Domain{ASCII: "reject.example"}, &reject, nil},
+		true, Result{false, StatusTemperror, false, false, dns.Domain{ASCII: "reject.example"}, &reject, false, nil},
 	)
 
 	// No reject if dkim temperror and no other pass.
@@ -253,7 +256,7 @@ func TestVerify(t *testing.T) {
 		},
 		spf.StatusNone,
 		nil,
-		true, Result{false, StatusTemperror, dns.Domain{ASCII: "reject.example"}, &reject, nil},
+		true, Result{false, StatusTemperror, false, false, dns.Domain{ASCII: "reject.example"}, &reject, false, nil},
 	)
 
 	// No reject if spf temperror but still dkim pass.
@@ -269,7 +272,7 @@ func TestVerify(t *testing.T) {
 		},
 		spf.StatusTemperror,
 		&dns.Domain{ASCII: "mail.reject.example"},
-		true, Result{false, StatusPass, dns.Domain{ASCII: "reject.example"}, &reject, nil},
+		true, Result{false, StatusPass, false, true, dns.Domain{ASCII: "reject.example"}, &reject, false, nil},
 	)
 
 	// No reject if dkim temperror but still spf pass.
@@ -285,7 +288,7 @@ func TestVerify(t *testing.T) {
 		},
 		spf.StatusPass,
 		&dns.Domain{ASCII: "mail.reject.example"},
-		true, Result{false, StatusPass, dns.Domain{ASCII: "reject.example"}, &reject, nil},
+		true, Result{false, StatusPass, true, false, dns.Domain{ASCII: "reject.example"}, &reject, false, nil},
 	)
 
 	// Bad DMARC record results in permerror without reject.
@@ -293,7 +296,7 @@ func TestVerify(t *testing.T) {
 		[]dkim.Result{},
 		spf.StatusNone,
 		nil,
-		false, Result{false, StatusPermerror, dns.Domain{ASCII: "malformed.example"}, nil, ErrSyntax},
+		false, Result{false, StatusPermerror, false, false, dns.Domain{ASCII: "malformed.example"}, nil, false, ErrSyntax},
 	)
 
 	// DKIM domain that is higher-level than organizational can not result in a pass. ../rfc/7489:525
@@ -309,6 +312,6 @@ func TestVerify(t *testing.T) {
 		},
 		spf.StatusNone,
 		nil,
-		true, Result{true, StatusFail, dns.Domain{ASCII: "example.com"}, &reject, nil},
+		true, Result{true, StatusFail, false, false, dns.Domain{ASCII: "example.com"}, &reject, false, nil},
 	)
 }
