@@ -57,6 +57,7 @@ any parameters. Followed by the help and usage information for each command.
 	mox help [command ...]
 	mox backup dest-dir
 	mox verifydata data-dir
+	mox licenses
 	mox config test
 	mox config dnscheck domain
 	mox config dnsrecords domain
@@ -695,6 +696,9 @@ recipients to be accepted, unless other reputation signals prevent that.
 Users can also import mailboxes/messages through the account web page by
 uploading a zip or tgz file with mbox and/or maildirs.
 
+Messages are imported even if already present. Importing messages twice will
+result in duplicate messages.
+
 Mailbox flags, like "seen", "answered", will be imported. An optional
 dovecot-keywords file can specify additional flags, like Forwarded/Junk/NotJunk.
 
@@ -722,6 +726,9 @@ recipients to be accepted, unless other reputation signals prevent that.
 Users can also import mailboxes/messages through the account web page by
 uploading a zip or tgz file with mbox and/or maildirs.
 
+Messages are imported even if already present. Importing messages twice will
+result in duplicate messages.
+
 	usage: mox import mbox accountname mailboxname mbox
 
 # mox export maildir
@@ -825,8 +832,12 @@ All files in the data directory that aren't recognized (i.e. other than known
 database files, message files, an acme directory, the "tmp" directory, etc),
 are stored, but with a warning.
 
-A clean successful backup does not print any output by default. Use the
--verbose flag for details, including timing.
+Remove files in the destination directory before doing another backup. The
+backup command will not overwrite files, but print and return errors.
+
+Exit code 0 indicates the backup was successful. A clean successful backup does
+not print any output, but may print warnings. Use the -verbose flag for
+details, including timing.
 
 To restore a backup, first shut down mox, move away the old data directory and
 move an earlier backed up directory in its place, run "mox verifydata",
@@ -870,6 +881,12 @@ possibly making them potentially no longer readable by the previous version.
 	  -skip-size-check
 	    	skip the check for message size
 
+# mox licenses
+
+Print licenses of mox source code and dependencies.
+
+	usage: mox licenses
+
 # mox config test
 
 Parses and validates the configuration files.
@@ -937,6 +954,8 @@ Remove an account and reload the configuration.
 Email addresses for this account will also be removed, and incoming email for
 these addresses will be rejected.
 
+All data for the account will be removed.
+
 	usage: mox config account rm account
 
 # mox config address add
