@@ -310,6 +310,7 @@ func (tc *testconn) close() {
 	}
 	err := tc.account.Close()
 	tc.check(err, "close account")
+	// no account.CheckClosed(), the tests open accounts multiple times.
 	tc.account = nil
 	tc.client.Close()
 	tc.serverConn.Close()
@@ -692,7 +693,7 @@ func DisabledTestReference(t *testing.T) {
 	defer tc3.close()
 	tc3.client.Login("mjl@mox.example", password0)
 	tc3.transactf("ok", `list "" "inbox" return (status (messages))`)
-	tc3.xuntagged(imapclient.UntaggedList{Separator: '/', Mailbox: "Inbox"}, imapclient.UntaggedStatus{Mailbox: "Inbox", Attrs: map[string]int64{"MESSAGES": 0}})
+	tc3.xuntagged(imapclient.UntaggedList{Separator: '/', Mailbox: "Inbox"}, imapclient.UntaggedStatus{Mailbox: "Inbox", Attrs: map[imapclient.StatusAttr]int64{imapclient.StatusMessages: 0}})
 
 	tc2.transactf("ok", "fetch 1 rfc822.size")
 	tc.xuntagged(imapclient.UntaggedFetch{Seq: 1, Attrs: []imapclient.FetchAttr{imapclient.FetchRFC822Size(len(exampleMsg))}})
