@@ -13,8 +13,8 @@ func TestSubscribe(t *testing.T) {
 	tc2 := startNoSwitchboard(t)
 	defer tc2.close()
 
-	tc.client.Login("mjl@mox.example", "testtest")
-	tc2.client.Login("mjl@mox.example", "testtest")
+	tc.client.Login("mjl@mox.example", password0)
+	tc2.client.Login("mjl@mox.example", password0)
 
 	tc.transactf("bad", "subscribe")       // Missing param.
 	tc.transactf("bad", "subscribe ")      // Missing param.
