@@ -63,6 +63,16 @@ var (
 
 var ErrConfig = errors.New("config error")
 
+// Set by packages webadmin, webaccount, webmail, webapisrv to prevent cyclic dependencies.
+var NewWebadminHandler = func(basePath string, isForwarded bool) http.Handler { return nopHandler }
+var NewWebaccountHandler = func(basePath string, isForwarded bool) http.Handler { return nopHandler }
+var NewWebmailHandler = func(maxMsgSize int64, basePath string, isForwarded bool, accountPath string) http.Handler {
+	return nopHandler
+}
+var NewWebapiHandler = func(maxMsgSize int64, basePath string, isForwarded bool) http.Handler { return nopHandler }
+
+var nopHandler = http.HandlerFunc(nil)
+
 // Config as used in the code, a processed version of what is in the config file.
 //
 // Use methods to lookup a domain/account/address in the dynamic configuration.
@@ -262,6 +272,13 @@ func (c *Config) Routes(accountName string, domain dns.Domain) (accountRoutes, d
 	return
 }
 
+func (c *Config) IsClientSettingsDomain(d dns.Domain) (is bool) {
+	c.withDynamicLock(func() {
+		_, is = c.Dynamic.ClientSettingDomains[d]
+	})
+	return
+}
+
 func (c *Config) allowACMEHosts(log mlog.Log, checkACMEHosts bool) {
 	for _, l := range c.Static.Listeners {
 		if l.TLS == nil || l.TLS.ACME == "" {
@@ -333,7 +350,11 @@ func (c *Config) allowACMEHosts(log mlog.Log, checkACMEHosts bool) {
 func writeDynamic(ctx context.Context, log mlog.Log, c config.Dynamic) error {
 	accDests, aliases, errs := prepareDynamicConfig(ctx, log, ConfigDynamicPath, Conf.Static, &c)
 	if len(errs) > 0 {
-		return fmt.Errorf("%w: %v", ErrConfig, errs[0])
+		errstrs := make([]string, len(errs))
+		for i, err := range errs {
+			errstrs[i] = err.Error()
+		}
+		return fmt.Errorf("%w: %s", ErrConfig, strings.Join(errstrs, "; "))
 	}
 
 	var b bytes.Buffer
@@ -688,28 +709,26 @@ func PrepareStaticConfig(ctx context.Context, log mlog.Log, configFile string, c
 
 				// If only checking or with missing ACME definition, we don't have an acme manager,
 				// so set an empty tls config to continue.
-				var tlsconfig *tls.Config
+				var tlsconfig, tlsconfigFallback *tls.Config
 				if checkOnly || acme.Manager == nil {
 					tlsconfig = &tls.Config{}
+					tlsconfigFallback = &tls.Config{}
 				} else {
-					tlsconfig = acme.Manager.TLSConfig.Clone()
-					l.TLS.ACMEConfig = acme.Manager.ACMETLSConfig
-
-					// SMTP STARTTLS connections are commonly made without SNI, because certificates
-					// often aren't verified.
 					hostname := c.HostnameDomain
 					if l.Hostname != "" {
 						hostname = l.HostnameDomain
 					}
-					getCert := tlsconfig.GetCertificate
-					tlsconfig.GetCertificate = func(hello *tls.ClientHelloInfo) (*tls.Certificate, error) {
-						if hello.ServerName == "" {
-							hello.ServerName = hostname.ASCII
-						}
-						return getCert(hello)
-					}
+					// If SNI is absent, we will use the listener hostname, but reject connections with
+					// an SNI hostname that is not allowlisted.
+					// Incoming SMTP deliveries use tlsconfigFallback for interoperability. TLS
+					// connections for unknown SNI hostnames fall back to a certificate for the
+					// listener hostname instead of causing the TLS connection to fail.
+					tlsconfig = acme.Manager.TLSConfig(hostname, true, false)
+					tlsconfigFallback = acme.Manager.TLSConfig(hostname, true, true)
+					l.TLS.ACMEConfig = acme.Manager.ACMETLSConfig
 				}
 				l.TLS.Config = tlsconfig
+				l.TLS.ConfigFallback = tlsconfigFallback
 			} else if len(l.TLS.KeyCerts) != 0 {
 				if doLoadTLSKeyCerts {
 					if err := loadTLSKeyCerts(configFile, "listener "+name, l.TLS); err != nil {
@@ -772,6 +791,9 @@ func PrepareStaticConfig(ctx context.Context, log mlog.Log, configFile string, c
 			if l.TLS.Config != nil {
 				l.TLS.Config.MinVersion = minVersion
 			}
+			if l.TLS.ConfigFallback != nil {
+				l.TLS.ConfigFallback.MinVersion = minVersion
+			}
 			if l.TLS.ACMEConfig != nil {
 				l.TLS.ACMEConfig.MinVersion = minVersion
 			}
@@ -1120,6 +1142,7 @@ func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string,
 	checkRoutes("global routes", c.Routes)
 
 	// Validate domains.
+	c.ClientSettingDomains = map[dns.Domain]struct{}{}
 	for d, domain := range c.Domains {
 		dnsdomain, err := dns.ParseDomain(d)
 		if err != nil {
@@ -1136,6 +1159,7 @@ func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string,
 				addErrorf("bad client settings domain %q: %s", domain.ClientSettingsDomain, err)
 			}
 			domain.ClientSettingsDNSDomain = csd
+			c.ClientSettingDomains[csd] = struct{}{}
 		}
 
 		for _, sign := range domain.DKIM.Sign {
@@ -1290,6 +1314,22 @@ func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string,
 			acc.NotJunkMailbox = r
 		}
 
+		if acc.JunkFilter != nil {
+			params := acc.JunkFilter.Params
+			if params.MaxPower < 0 || params.MaxPower > 0.5 {
+				addErrorf("junk filter MaxPower must be >= 0 and < 0.5")
+			}
+			if params.TopWords < 0 {
+				addErrorf("junk filter TopWords must be >= 0")
+			}
+			if params.IgnoreWords < 0 || params.IgnoreWords > 0.5 {
+				addErrorf("junk filter IgnoreWords must be >= 0 and < 0.5")
+			}
+			if params.RareWords < 0 {
+				addErrorf("junk filter RareWords must be >= 0")
+			}
+		}
+
 		acc.ParsedFromIDLoginAddresses = make([]smtp.Address, len(acc.FromIDLoginAddresses))
 		for i, s := range acc.FromIDLoginAddresses {
 			a, err := smtp.ParseAddress(s)
@@ -1794,6 +1834,29 @@ func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string,
 				}
 			}
 		}
+		if wh.WebInternal != nil {
+			n++
+			wi := wh.WebInternal
+			if !strings.HasPrefix(wi.BasePath, "/") || !strings.HasSuffix(wi.BasePath, "/") {
+				addErrorf("webinternal %s %s: base path %q must start and end with /", wh.Domain, wh.PathRegexp, wi.BasePath)
+			}
+			// todo: we could make maxMsgSize and accountPath configurable
+			const isForwarded = false
+			switch wi.Service {
+			case "admin":
+				wi.Handler = NewWebadminHandler(wi.BasePath, isForwarded)
+			case "account":
+				wi.Handler = NewWebaccountHandler(wi.BasePath, isForwarded)
+			case "webmail":
+				accountPath := ""
+				wi.Handler = NewWebmailHandler(config.DefaultMaxMsgSize, wi.BasePath, isForwarded, accountPath)
+			case "webapi":
+				wi.Handler = NewWebapiHandler(config.DefaultMaxMsgSize, wi.BasePath, isForwarded)
+			default:
+				addErrorf("webinternal %s %s: unknown service %q", wh.Domain, wh.PathRegexp, wi.Service)
+			}
+			wi.Handler = SafeHeaders(http.StripPrefix(wi.BasePath[:len(wi.BasePath)-1], wi.Handler))
+		}
 		if n != 1 {
 			addErrorf("webhandler %s %s: must have exactly one handler, not %d", wh.Domain, wh.PathRegexp, n)
 		}
@@ -1859,6 +1922,7 @@ func loadTLSKeyCerts(configFile, kind string, ctls *config.TLS) error {
 	ctls.Config = &tls.Config{
 		Certificates: certs,
 	}
+	ctls.ConfigFallback = ctls.Config
 	return nil
 }
 
