@@ -6,6 +6,7 @@ import (
 	"crypto/sha256"
 	"fmt"
 	"io"
+	"log/slog"
 	"os"
 
 	"github.com/mjl-/bstore"
@@ -17,15 +18,15 @@ import (
 )
 
 // rejectPresent returns whether the message is already present in the rejects mailbox.
-func rejectPresent(log *mlog.Log, acc *store.Account, rejectsMailbox string, m *store.Message, f *os.File) (present bool, msgID string, hash []byte, rerr error) {
-	if p, err := message.Parse(log, false, store.FileMsgReader(m.MsgPrefix, f)); err != nil {
+func rejectPresent(log mlog.Log, acc *store.Account, rejectsMailbox string, m *store.Message, f *os.File) (present bool, msgID string, hash []byte, rerr error) {
+	if p, err := message.Parse(log.Logger, false, store.FileMsgReader(m.MsgPrefix, f)); err != nil {
 		log.Infox("parsing reject message for message-id", err)
 	} else if header, err := p.Header(); err != nil {
 		log.Infox("parsing reject message header for message-id", err)
 	} else {
 		msgID, _, err = message.MessageIDCanonical(header.Get("Message-Id"))
 		if err != nil {
-			log.Debugx("parsing message-id for reject", err, mlog.Field("messageid", header.Get("Message-Id")))
+			log.Debugx("parsing message-id for reject", err, slog.String("messageid", header.Get("Message-Id")))
 		}
 	}
 
