@@ -108,7 +108,8 @@ func TestReputation(t *testing.T) {
 		p := filepath.FromSlash("../testdata/smtpserver-reputation.db")
 		defer os.Remove(p)
 
-		db, err := bstore.Open(ctxbg, p, &bstore.Options{Timeout: 5 * time.Second}, store.DBTypes...)
+		opts := bstore.Options{Timeout: 5 * time.Second, RegisterLogger: log.Logger}
+		db, err := bstore.Open(ctxbg, p, &opts, store.DBTypes...)
 		tcheck(t, err, "open db")
 		defer db.Close()
 
@@ -147,7 +148,7 @@ func TestReputation(t *testing.T) {
 		var method reputationMethod
 		err = db.Read(ctxbg, func(tx *bstore.Tx) error {
 			var err error
-			isjunk, conclusive, method, err = reputation(tx, pkglog, &m)
+			isjunk, conclusive, method, _, err = reputation(tx, pkglog, &m, false)
 			return err
 		})
 		tcheck(t, err, "read tx")
