@@ -16,39 +16,22 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
+	"log/slog"
 	"net/http"
 	"strconv"
 	"strings"
 	"time"
 
-	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/client_golang/prometheus/promauto"
-
 	"github.com/mjl-/mox/dns"
-	"github.com/mjl-/mox/metrics"
 	"github.com/mjl-/mox/mlog"
 	"github.com/mjl-/mox/moxio"
+	"github.com/mjl-/mox/stub"
 )
 
-var xlog = mlog.New("updates")
-
 var (
-	metricLookup = promauto.NewHistogramVec(
-		prometheus.HistogramOpts{
-			Name:    "mox_updates_lookup_duration_seconds",
-			Help:    "Updates lookup with result.",
-			Buckets: []float64{0.001, 0.005, 0.01, 0.05, 0.100, 0.5, 1, 5, 10, 20, 30},
-		},
-		[]string{"result"},
-	)
-	metricFetchChangelog = promauto.NewHistogramVec(
-		prometheus.HistogramOpts{
-			Name:    "mox_updates_fetchchangelog_duration_seconds",
-			Help:    "Fetch changelog with result.",
-			Buckets: []float64{0.001, 0.005, 0.01, 0.05, 0.100, 0.5, 1, 5, 10, 20, 30},
-		},
-		[]string{"result"},
-	)
+	MetricLookup         stub.HistogramVec                                                                                           = stub.HistogramVecIgnore{}
+	MetricFetchChangelog stub.HistogramVec                                                                                           = stub.HistogramVecIgnore{}
+	HTTPClientObserve    func(ctx context.Context, log *slog.Logger, pkg, method string, statusCode int, err error, start time.Time) = stub.HTTPClientObserveIgnore
 )
 
 var (
@@ -81,22 +64,26 @@ type Changelog struct {
 
 // Lookup looks up the updates DNS TXT record at "_updates.<domain>" and returns
 // the parsed form.
-func Lookup(ctx context.Context, resolver dns.Resolver, domain dns.Domain) (rversion Version, rrecord *Record, rerr error) {
-	log := xlog.WithContext(ctx)
+func Lookup(ctx context.Context, elog *slog.Logger, resolver dns.Resolver, domain dns.Domain) (rversion Version, rrecord *Record, rerr error) {
+	log := mlog.New("updates", elog)
 	start := time.Now()
 	defer func() {
 		var result = "ok"
 		if rerr != nil {
 			result = "error"
 		}
-		metricLookup.WithLabelValues(result).Observe(float64(time.Since(start)) / float64(time.Second))
-		log.Debugx("updates lookup result", rerr, mlog.Field("domain", domain), mlog.Field("version", rversion), mlog.Field("record", rrecord), mlog.Field("duration", time.Since(start)))
+		MetricLookup.ObserveLabels(float64(time.Since(start))/float64(time.Second), result)
+		log.Debugx("updates lookup result", rerr,
+			slog.Any("domain", domain),
+			slog.Any("version", rversion),
+			slog.Any("record", rrecord),
+			slog.Duration("duration", time.Since(start)))
 	}()
 
 	nctx, cancel := context.WithTimeout(ctx, 30*time.Second)
 	defer cancel()
 	name := "_updates." + domain.ASCII + "."
-	txts, err := dns.WithPackage(resolver, "updates").LookupTXT(nctx, name)
+	txts, _, err := dns.WithPackage(resolver, "updates").LookupTXT(nctx, name)
 	if dns.IsNotFound(err) {
 		return Version{}, nil, ErrNoRecord
 	} else if err != nil {
@@ -132,16 +119,19 @@ func Lookup(ctx context.Context, resolver dns.Resolver, domain dns.Domain) (rver
 // error is returned.
 //
 // A changelog can be maximum 1 MB.
-func FetchChangelog(ctx context.Context, baseURL string, base Version, pubKey []byte) (changelog *Changelog, rerr error) {
-	log := xlog.WithContext(ctx)
+func FetchChangelog(ctx context.Context, elog *slog.Logger, baseURL string, base Version, pubKey []byte) (changelog *Changelog, rerr error) {
+	log := mlog.New("updates", elog)
 	start := time.Now()
 	defer func() {
 		var result = "ok"
 		if rerr != nil {
 			result = "error"
 		}
-		metricFetchChangelog.WithLabelValues(result).Observe(float64(time.Since(start)) / float64(time.Second))
-		log.Debugx("updates fetch changelog result", rerr, mlog.Field("baseurl", baseURL), mlog.Field("base", base), mlog.Field("duration", time.Since(start)))
+		MetricFetchChangelog.ObserveLabels(float64(time.Since(start))/float64(time.Second), result)
+		log.Debugx("updates fetch changelog result", rerr,
+			slog.String("baseurl", baseURL),
+			slog.Any("base", base),
+			slog.Duration("duration", time.Since(start)))
 	}()
 
 	url := baseURL + "?from=" + base.String()
@@ -156,7 +146,7 @@ func FetchChangelog(ctx context.Context, baseURL string, base Version, pubKey []
 	if resp == nil {
 		resp = &http.Response{StatusCode: 0}
 	}
-	metrics.HTTPClientObserve(ctx, "updates", req.Method, resp.StatusCode, err, start)
+	HTTPClientObserve(ctx, log.Logger, "updates", req.Method, resp.StatusCode, err, start)
 	if err != nil {
 		return nil, fmt.Errorf("%w: making http request: %s", ErrChangelogFetch, err)
 	}
@@ -182,20 +172,34 @@ func FetchChangelog(ctx context.Context, baseURL string, base Version, pubKey []
 
 // Check checks for an updated version through DNS and fetches a
 // changelog if so.
-func Check(ctx context.Context, resolver dns.Resolver, domain dns.Domain, lastKnown Version, changelogBaseURL string, pubKey []byte) (rversion Version, rrecord *Record, changelog *Changelog, rerr error) {
-	log := xlog.WithContext(ctx)
+//
+// Check looks up a TXT record at _updates.<domain>, and parses the record. If the
+// latest version is more recent than lastKnown, an update is available, and Check
+// will fetch the signed changes since lastKnown, verify the signatures, and
+// return the changelog. The latest version and parsed DNS record is returned
+// regardless of whether a new version was found. A non-nil changelog is only
+// returned when a new version was found and a changelog could be fetched and
+// verified.
+func Check(ctx context.Context, elog *slog.Logger, resolver dns.Resolver, domain dns.Domain, lastKnown Version, changelogBaseURL string, pubKey []byte) (rversion Version, rrecord *Record, changelog *Changelog, rerr error) {
+	log := mlog.New("updates", elog)
 	start := time.Now()
 	defer func() {
-		log.Debugx("updates check result", rerr, mlog.Field("domain", domain), mlog.Field("lastknown", lastKnown), mlog.Field("changelogbaseurl", changelogBaseURL), mlog.Field("version", rversion), mlog.Field("record", rrecord), mlog.Field("duration", time.Since(start)))
+		log.Debugx("updates check result", rerr,
+			slog.Any("domain", domain),
+			slog.Any("lastknown", lastKnown),
+			slog.String("changelogbaseurl", changelogBaseURL),
+			slog.Any("version", rversion),
+			slog.Any("record", rrecord),
+			slog.Duration("duration", time.Since(start)))
 	}()
 
-	latest, record, err := Lookup(ctx, resolver, domain)
+	latest, record, err := Lookup(ctx, log.Logger, resolver, domain)
 	if err != nil {
 		return latest, record, nil, err
 	}
 
 	if latest.After(lastKnown) {
-		changelog, err = FetchChangelog(ctx, changelogBaseURL, lastKnown, pubKey)
+		changelog, err = FetchChangelog(ctx, log.Logger, changelogBaseURL, lastKnown, pubKey)
 	}
 	return latest, record, changelog, err
 }
