@@ -85,6 +85,10 @@ func init() {
 	if err != nil {
 		pkglog.Fatalx("sherpa handler", err)
 	}
+
+	mox.NewWebaccountHandler = func(basePath string, isForwarded bool) http.Handler {
+		return http.HandlerFunc(Handler(basePath, isForwarded))
+	}
 }
 
 // Handler returns a handler for the webaccount endpoints, customized for the
@@ -104,6 +108,11 @@ func xcheckf(ctx context.Context, err error, format string, args ...any) {
 	if err == nil {
 		return
 	}
+	// If caller tried saving a config that is invalid, or because of a bad request, cause a user error.
+	if errors.Is(err, mox.ErrConfig) || errors.Is(err, mox.ErrRequest) {
+		xcheckuserf(ctx, err, format, args...)
+	}
+
 	msg := fmt.Sprintf(format, args...)
 	errmsg := fmt.Sprintf("%s: %s", msg, err)
 	pkglog.WithContext(ctx).Errorx(msg, err)
@@ -202,6 +211,14 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 			http.Error(w, "405 - method not allowed - use get", http.StatusMethodNotAllowed)
 		}
 		return
+	} else if r.URL.Path == "/licenses.txt" {
+		switch r.Method {
+		case "GET", "HEAD":
+			mox.LicensesWrite(w)
+		default:
+			http.Error(w, "405 - method not allowed - use get", http.StatusMethodNotAllowed)
+		}
+		return
 	}
 
 	isAPI := strings.HasPrefix(r.URL.Path, "/api/")
@@ -516,9 +533,6 @@ func (Account) OutgoingWebhookSave(ctx context.Context, url, authorization strin
 			acc.OutgoingWebhook = &config.OutgoingWebhook{URL: url, Authorization: authorization, Events: events}
 		}
 	})
-	if err != nil && errors.Is(err, mox.ErrConfig) {
-		xcheckuserf(ctx, err, "saving account outgoing webhook")
-	}
 	xcheckf(ctx, err, "saving account outgoing webhook")
 }
 
@@ -558,9 +572,6 @@ func (Account) IncomingWebhookSave(ctx context.Context, url, authorization strin
 			acc.IncomingWebhook = &config.IncomingWebhook{URL: url, Authorization: authorization}
 		}
 	})
-	if err != nil && errors.Is(err, mox.ErrConfig) {
-		xcheckuserf(ctx, err, "saving account incoming webhook")
-	}
 	xcheckf(ctx, err, "saving account incoming webhook")
 }
 
@@ -602,9 +613,6 @@ func (Account) FromIDLoginAddressesSave(ctx context.Context, loginAddresses []st
 	err := mox.AccountSave(ctx, reqInfo.AccountName, func(acc *config.Account) {
 		acc.FromIDLoginAddresses = loginAddresses
 	})
-	if err != nil && errors.Is(err, mox.ErrConfig) {
-		xcheckuserf(ctx, err, "saving account fromid login addresses")
-	}
 	xcheckf(ctx, err, "saving account fromid login addresses")
 }
 
@@ -615,9 +623,6 @@ func (Account) KeepRetiredPeriodsSave(ctx context.Context, keepRetiredMessagePer
 		acc.KeepRetiredMessagePeriod = keepRetiredMessagePeriod
 		acc.KeepRetiredWebhookPeriod = keepRetiredWebhookPeriod
 	})
-	if err != nil && errors.Is(err, mox.ErrConfig) {
-		xcheckuserf(ctx, err, "saving account keep retired periods")
-	}
 	xcheckf(ctx, err, "saving account keep retired periods")
 }
 
@@ -633,12 +638,28 @@ func (Account) AutomaticJunkFlagsSave(ctx context.Context, enabled bool, junkReg
 			NotJunkMailboxRegexp: notJunkRegexp,
 		}
 	})
-	if err != nil && errors.Is(err, mox.ErrConfig) {
-		xcheckuserf(ctx, err, "saving account automatic junk flags")
-	}
 	xcheckf(ctx, err, "saving account automatic junk flags")
 }
 
+// JunkFilterSave saves junk filter settings. If junkFilter is nil, the junk filter
+// is disabled. Otherwise all fields except Threegrams are stored.
+func (Account) JunkFilterSave(ctx context.Context, junkFilter *config.JunkFilter) {
+	reqInfo := ctx.Value(requestInfoCtxKey).(requestInfo)
+	err := mox.AccountSave(ctx, reqInfo.AccountName, func(acc *config.Account) {
+		if junkFilter == nil {
+			acc.JunkFilter = nil
+			return
+		}
+		old := acc.JunkFilter
+		acc.JunkFilter = junkFilter
+		acc.JunkFilter.Params.Threegrams = false
+		if old != nil {
+			acc.JunkFilter.Params.Threegrams = old.Params.Threegrams
+		}
+	})
+	xcheckf(ctx, err, "saving account junk filter settings")
+}
+
 // RejectsSave saves the RejectsMailbox and KeepRejects settings.
 func (Account) RejectsSave(ctx context.Context, mailbox string, keep bool) {
 	reqInfo := ctx.Value(requestInfoCtxKey).(requestInfo)
@@ -646,8 +667,5 @@ func (Account) RejectsSave(ctx context.Context, mailbox string, keep bool) {
 		acc.RejectsMailbox = mailbox
 		acc.KeepRejects = keep
 	})
-	if err != nil && errors.Is(err, mox.ErrConfig) {
-		xcheckuserf(ctx, err, "saving account rejects settings")
-	}
 	xcheckf(ctx, err, "saving account rejects settings")
 }
