@@ -28,6 +28,7 @@ import (
 
 	"github.com/mjl-/mox/config"
 	"github.com/mjl-/mox/dns"
+	"github.com/mjl-/mox/junk"
 	"github.com/mjl-/mox/mlog"
 	"github.com/mjl-/mox/mox-"
 	"github.com/mjl-/mox/queue"
@@ -229,6 +230,7 @@ func TestAccount(t *testing.T) {
 
 	err = queue.Init() // For DB.
 	tcheck(t, err, "queue init")
+	defer queue.Shutdown()
 
 	account, _, _, _ := api.Account(ctx)
 
@@ -249,6 +251,9 @@ func TestAccount(t *testing.T) {
 	api.AccountSaveFullName(ctx, account.FullName)
 
 	go ImportManage()
+	defer func() {
+		importers.Stop <- struct{}{}
+	}()
 
 	// Import mbox/maildir tgz/zip.
 	testImport := func(filename string, expect int) {
@@ -463,6 +468,18 @@ func TestAccount(t *testing.T) {
 	api.AutomaticJunkFlagsSave(ctx, true, "^(junk|spam)", "^(inbox|neutral|postmaster|dmarc|tlsrpt|rejects)", "")
 	api.AutomaticJunkFlagsSave(ctx, false, "", "", "")
 
+	api.JunkFilterSave(ctx, nil)
+	jf := config.JunkFilter{
+		Threshold: 0.95,
+		Params: junk.Params{
+			Twograms:    true,
+			MaxPower:    0.1,
+			TopWords:    10,
+			IgnoreWords: 0.1,
+		},
+	}
+	api.JunkFilterSave(ctx, &jf)
+
 	api.RejectsSave(ctx, "Rejects", true)
 	api.RejectsSave(ctx, "Rejects", false)
 	api.RejectsSave(ctx, "", false) // Restore.
