@@ -252,6 +252,10 @@ fieldset { border: 0; }
 		panic("executing api docs index template: " + err.Error())
 	}
 	docsIndex = b.Bytes()
+
+	mox.NewWebapiHandler = func(maxMsgSize int64, basePath string, isForwarded bool) http.Handler {
+		return NewServer(maxMsgSize, basePath, isForwarded)
+	}
 }
 
 // NewServer returns a new http.Handler for a webapi server.
@@ -755,8 +759,9 @@ func (s server) Send(ctx context.Context, req webapi.SendRequest) (resp webapi.S
 		xc.Header("User-Agent", "mox/"+moxvar.Version)
 	}
 
-	// Whether we have additional separately inline/attached file(s).
+	// Whether we have additional separately alternative/inline/attached file(s).
 	mpf := reqInfo.Request.MultipartForm
+	formAlternative := mpf != nil && len(mpf.File["alternativefile"]) > 0
 	formInline := mpf != nil && len(mpf.File["inlinefile"]) > 0
 	formAttachment := mpf != nil && len(mpf.File["attachedfile"]) > 0
 
@@ -766,6 +771,7 @@ func (s server) Send(ctx context.Context, req webapi.SendRequest) (resp webapi.S
 	//     - multipart/alternative (in case we have both text and html bodies)
 	//       - text/plain (optional)
 	//       - text/html (optional)
+	//       - alternative file, ...
 	//     - inline file, ...
 	//   - attached file, ...
 
@@ -807,7 +813,7 @@ func (s server) Send(ctx context.Context, req webapi.SendRequest) (resp webapi.S
 		related = xcreateMultipart("related")
 		cur = related
 	}
-	if m.Text != "" && m.HTML != "" {
+	if m.Text != "" && m.HTML != "" || len(req.AlternativeFiles) > 0 || formAlternative {
 		alternative = xcreateMultipart("alternative")
 		cur = alternative
 	}
@@ -823,10 +829,6 @@ func (s server) Send(ctx context.Context, req webapi.SendRequest) (resp webapi.S
 		_, err := tp.Write([]byte(htmlBody))
 		xcheckf(err, "write html part")
 	}
-	if alternative != nil {
-		alternative.Close()
-		alternative = nil
-	}
 
 	xaddFileBase64 := func(ct string, inline bool, filename string, cid string, base64Data string) {
 		h := textproto.MIMEHeader{}
@@ -919,6 +921,18 @@ func (s server) Send(ctx context.Context, req webapi.SendRequest) (resp webapi.S
 		xcheckf(err, "flushing uploaded file")
 	}
 
+	cur = alternative
+	xaddJSONFiles(req.AlternativeFiles, true)
+	if mpf != nil {
+		for _, fh := range mpf.File["alternativefile"] {
+			xaddFile(fh, true)
+		}
+	}
+	if alternative != nil {
+		alternative.Close()
+		alternative = nil
+	}
+
 	cur = related
 	xaddJSONFiles(req.InlineFiles, true)
 	if mpf != nil {
@@ -1229,7 +1243,11 @@ func (s server) MessageGet(ctx context.Context, req webapi.MessageGetRequest) (r
 
 	var msgFrom string
 	if d, err := dns.ParseDomain(m.MsgFromDomain); err == nil {
-		msgFrom = smtp.Address{Localpart: m.MsgFromLocalpart, Domain: d}.Pack(true)
+		msgFrom = smtp.NewAddress(m.MsgFromLocalpart, d).Pack(true)
+	}
+	var rcptTo string
+	if m.RcptToDomain != "" {
+		rcptTo = m.RcptToLocalpart.String() + "@" + m.RcptToDomain
 	}
 	meta := webapi.MessageMeta{
 		Size:                m.Size,
@@ -1237,6 +1255,7 @@ func (s server) MessageGet(ctx context.Context, req webapi.MessageGetRequest) (r
 		Flags:               append(m.Flags.Strings(), m.Keywords...),
 		MailFrom:            m.MailFrom,
 		MailFromValidated:   m.MailFromValidated,
+		RcptTo:              rcptTo,
 		MsgFrom:             msgFrom,
 		MsgFromValidated:    m.MsgFromValidated,
 		DKIMVerifiedDomains: m.DKIMDomains,
