@@ -130,9 +130,10 @@ func (w Webmail) Logout(ctx context.Context) {
 	xcheckf(ctx, err, "logout")
 }
 
-// Token returns a token to use for an SSE connection. A token can only be used for
-// a single SSE connection. Tokens are stored in memory for a maximum of 1 minute,
-// with at most 10 unused tokens (the most recently created) per account.
+// Token returns a single-use token to use for an SSE connection. A token can only
+// be used for a single SSE connection. Tokens are stored in memory for a maximum
+// of 1 minute, with at most 10 unused tokens (the most recently created) per
+// account.
 func (Webmail) Token(ctx context.Context) string {
 	reqInfo := ctx.Value(requestInfoCtxKey).(requestInfo)
 	return sseTokens.xgenerate(ctx, reqInfo.Account.Name, reqInfo.LoginAddress, reqInfo.SessionToken)
@@ -183,17 +184,30 @@ func (Webmail) ParsedMessage(ctx context.Context, msgID int64) (pm ParsedMessage
 
 // fromAddrViewMode returns the view mode for a from address.
 func fromAddrViewMode(tx *bstore.Tx, from MessageAddress) (store.ViewMode, error) {
+	settingsViewMode := func() (store.ViewMode, error) {
+		settings := store.Settings{ID: 1}
+		if err := tx.Get(&settings); err != nil {
+			return store.ModeText, err
+		}
+		if settings.ShowHTML {
+			return store.ModeHTML, nil
+		}
+		return store.ModeText, nil
+	}
+
 	lp, err := smtp.ParseLocalpart(from.User)
 	if err != nil {
-		return store.ModeDefault, nil
+		return settingsViewMode()
 	}
-	fromAddr := smtp.Address{Localpart: lp, Domain: from.Domain}.Pack(true)
+	fromAddr := smtp.NewAddress(lp, from.Domain).Pack(true)
 	fas := store.FromAddressSettings{FromAddress: fromAddr}
 	err = tx.Get(&fas)
 	if err == bstore.ErrAbsent {
-		return store.ModeDefault, nil
+		return settingsViewMode()
+	} else if err != nil {
+		return store.ModeText, err
 	}
-	return fas.ViewMode, err
+	return fas.ViewMode, nil
 }
 
 // FromAddressSettingsSave saves per-"From"-address settings.
@@ -517,13 +531,14 @@ type File struct {
 // parseAddress expects either a plain email address like "user@domain", or a
 // single address as used in a message header, like "name <user@domain>".
 func parseAddress(msghdr string) (message.NameAddress, error) {
-	a, err := mail.ParseAddress(msghdr)
+	// todo: parse more fully according to ../rfc/5322:959
+	parser := mail.AddressParser{WordDecoder: &wordDecoder}
+	a, err := parser.Parse(msghdr)
 	if err != nil {
 		return message.NameAddress{}, err
 	}
 
-	// todo: parse more fully according to ../rfc/5322:959
-	path, err := smtp.ParseAddress(a.Address)
+	path, err := smtp.ParseNetMailAddress(a.Address)
 	if err != nil {
 		return message.NameAddress{}, err
 	}
@@ -1658,12 +1673,12 @@ func recipientSecurity(ctx context.Context, log mlog.Log, resolver dns.Resolver,
 		SecurityResultUnknown,
 	}
 
-	msgAddr, err := mail.ParseAddress(messageAddressee)
+	parser := mail.AddressParser{WordDecoder: &wordDecoder}
+	msgAddr, err := parser.Parse(messageAddressee)
 	if err != nil {
-		return rs, fmt.Errorf("parsing message addressee: %v", err)
+		return rs, fmt.Errorf("parsing addressee: %v", err)
 	}
-
-	addr, err := smtp.ParseAddress(msgAddr.Address)
+	addr, err := smtp.ParseNetMailAddress(msgAddr.Address)
 	if err != nil {
 		return rs, fmt.Errorf("parsing address: %v", err)
 	}
