@@ -17,6 +17,7 @@ import (
 	"github.com/mjl-/mox/dns"
 	"github.com/mjl-/mox/mlog"
 	"github.com/mjl-/mox/mox-"
+	"github.com/mjl-/mox/mtastsdb"
 	"github.com/mjl-/mox/queue"
 	"github.com/mjl-/mox/store"
 )
@@ -58,6 +59,8 @@ func TestAPI(t *testing.T) {
 	defer store.Switchboard()()
 
 	log := mlog.New("webmail", nil)
+	err := mtastsdb.Init(false)
+	tcheck(t, err, "mtastsdb init")
 	acc, err := store.OpenAccount(log, "mjl")
 	tcheck(t, err, "open account")
 	const pw0 = "te\u0301st \u00a0\u2002\u200a" // NFD and various unicode spaces.
@@ -65,7 +68,9 @@ func TestAPI(t *testing.T) {
 	err = acc.SetPassword(log, pw0)
 	tcheck(t, err, "set password")
 	defer func() {
-		err := acc.Close()
+		err := mtastsdb.Close()
+		tcheck(t, err, "mtastsdb close")
+		err = acc.Close()
 		pkglog.Check(err, "closing account")
 		acc.CheckClosed()
 	}()
@@ -227,7 +232,7 @@ func TestAPI(t *testing.T) {
 	tneedError(t, func() { api.ParsedMessage(ctx, 0) })
 	tneedError(t, func() { api.ParsedMessage(ctx, testmsgs[len(testmsgs)-1].ID+1) })
 	pm := api.ParsedMessage(ctx, inboxText.ID)
-	tcompare(t, pm.ViewMode, store.ModeDefault)
+	tcompare(t, pm.ViewMode, store.ModeText)
 
 	api.FromAddressSettingsSave(ctx, store.FromAddressSettings{FromAddress: "mjl@mox.example", ViewMode: store.ModeHTMLExt})
 	pm = api.ParsedMessage(ctx, inboxText.ID)
@@ -309,6 +314,7 @@ func TestAPI(t *testing.T) {
 	queue.Localserve = true // Deliver directly to us instead attempting actual delivery.
 	err = queue.Init()
 	tcheck(t, err, "queue init")
+	defer queue.Shutdown()
 	api.MessageSubmit(ctx, SubmitMessage{
 		From:           "mjl@mox.example",
 		To:             []string{"mjl+to@mox.example", "mjl to2 <mjl+to2@mox.example>"},
