@@ -170,6 +170,12 @@ func serveContentFallback(log mlog.Log, w http.ResponseWriter, r *http.Request,
 	http.ServeContent(w, r, "", mox.FallbackMtime(log), bytes.NewReader(fallback))
 }
 
+func init() {
+	mox.NewWebmailHandler = func(maxMsgSize int64, basePath string, isForwarded bool, accountPath string) http.Handler {
+		return http.HandlerFunc(Handler(maxMsgSize, basePath, isForwarded, accountPath))
+	}
+}
+
 // Handler returns a handler for the webmail endpoints, customized for the max
 // message size coming from the listener and cookiePath.
 func Handler(maxMessageSize int64, cookiePath string, isForwarded bool, accountPath string) func(w http.ResponseWriter, r *http.Request) {
@@ -188,8 +194,8 @@ func handle(apiHandler http.Handler, isForwarded bool, accountPath string, w htt
 	log := pkglog.WithContext(ctx).With(slog.String("userauth", ""))
 
 	// Server-sent event connection, for all initial data (list of mailboxes), list of
-	// messages, and all events afterwards. Authenticated through a token in the query
-	// string, which it got from a Token API call.
+	// messages, and all events afterwards. Authenticated through a single use token in
+	// the query string, which it got from a Token API call.
 	if r.URL.Path == "/events" {
 		serveEvents(ctx, log, accountPath, w, r)
 		return
@@ -229,6 +235,17 @@ func handle(apiHandler http.Handler, isForwarded bool, accountPath string, w htt
 		}
 		return
 
+	case "/licenses.txt":
+		switch r.Method {
+		case "GET", "HEAD":
+			h := w.Header()
+			h.Set("Content-Type", "text/plain; charset=utf-8")
+			mox.LicensesWrite(w)
+		default:
+			http.Error(w, "405 - method not allowed - use get", http.StatusMethodNotAllowed)
+		}
+		return
+
 	case "/msg.js", "/text.js":
 		switch r.Method {
 		default:
