@@ -1,8 +1,10 @@
 package webmail
 
 import (
+	"archive/tar"
 	"archive/zip"
 	"bytes"
+	"compress/gzip"
 	"context"
 	"encoding/json"
 	"fmt"
@@ -11,6 +13,7 @@ import (
 	"net/http"
 	"net/http/httptest"
 	"net/textproto"
+	"net/url"
 	"os"
 	"path/filepath"
 	"reflect"
@@ -23,6 +26,7 @@ import (
 	"github.com/mjl-/sherpa"
 
 	"github.com/mjl-/mox/message"
+	"github.com/mjl-/mox/mlog"
 	"github.com/mjl-/mox/mox-"
 	"github.com/mjl-/mox/moxio"
 	"github.com/mjl-/mox/store"
@@ -188,6 +192,7 @@ var (
 		From:    "mjl <mjl@mox.example>",
 		To:      "mox <mox@other.example>",
 		Subject: "html message",
+		Headers: [][2]string{{"List-Id", "test <list.mox.example>"}},
 		Part:    Part{Type: "text/html", Content: `<html>the body <img src="cid:img1@mox.example" /></html>`},
 	}
 	msgAlt = Message{
@@ -265,7 +270,16 @@ func tdeliver(t *testing.T, acc *store.Account, tm *testmsg) {
 	defer msgFile.Close()
 	size, err := msgFile.Write(tm.msg.Marshal(t))
 	tcheck(t, err, "write message temp")
-	m := store.Message{Flags: tm.Flags, Keywords: tm.Keywords, Size: int64(size)}
+	m := store.Message{
+		Flags:            tm.Flags,
+		RcptToLocalpart:  "mox",
+		RcptToDomain:     "other.example",
+		MsgFromLocalpart: "mjl",
+		MsgFromDomain:    "mox.example",
+		DKIMDomains:      []string{"mox.example"},
+		Keywords:         tm.Keywords,
+		Size:             int64(size),
+	}
 	err = acc.DeliverMailbox(pkglog, tm.Mailbox, &m, msgFile)
 	tcheck(t, err, "deliver test message")
 	err = msgFile.Close()
@@ -294,6 +308,7 @@ func TestWebmail(t *testing.T) {
 	mox.MustLoadConfig(true, false)
 	defer store.Switchboard()()
 
+	log := mlog.New("webmail", nil)
 	acc, err := store.OpenAccount(pkglog, "mjl")
 	tcheck(t, err, "open account")
 	err = acc.SetPassword(pkglog, "test1234")
@@ -301,6 +316,7 @@ func TestWebmail(t *testing.T) {
 	defer func() {
 		err := acc.Close()
 		pkglog.Check(err, "closing account")
+		acc.CheckClosed()
 	}()
 
 	api := Webmail{maxMessageSize: 1024 * 1024, cookiePath: "/webmail/"}
@@ -308,7 +324,7 @@ func TestWebmail(t *testing.T) {
 	tcheck(t, err, "sherpa handler")
 
 	respRec := httptest.NewRecorder()
-	reqInfo := requestInfo{"", "", "", respRec, &http.Request{RemoteAddr: "127.0.0.1:1234"}}
+	reqInfo := requestInfo{log, "", nil, "", respRec, &http.Request{RemoteAddr: "127.0.0.1:1234"}}
 	ctx := context.WithValue(ctxbg, requestInfoCtxKey, reqInfo)
 
 	// Prepare loginToken.
@@ -328,7 +344,7 @@ func TestWebmail(t *testing.T) {
 		t.Fatalf("missing session cookie")
 	}
 
-	reqInfo = requestInfo{"mjl@mox.example", "mjl", "", respRec, &http.Request{RemoteAddr: "127.0.0.1:1234"}}
+	reqInfo = requestInfo{log, "mjl@mox.example", acc, "", respRec, &http.Request{RemoteAddr: "127.0.0.1:1234"}}
 	ctx = context.WithValue(ctxbg, requestInfoCtxKey, reqInfo)
 
 	tneedError(t, func() { api.MailboxCreate(ctx, "Inbox") })   // Cannot create inbox.
@@ -376,7 +392,7 @@ func TestWebmail(t *testing.T) {
 		}
 		rr := httptest.NewRecorder()
 		rr.Body = &bytes.Buffer{}
-		handle(apiHandler, false, rr, req)
+		handle(apiHandler, false, "", rr, req)
 		if rr.Code != expStatusCode {
 			t.Fatalf("got status %d, expected %d (%s)", rr.Code, expStatusCode, readBody(rr.Body))
 		}
@@ -449,6 +465,74 @@ func TestWebmail(t *testing.T) {
 	// Unknown.
 	testHTTP("GET", "/other", httpHeaders{}, http.StatusForbidden, nil, nil)
 
+	// Export.
+	testHTTP("GET", "/export", httpHeaders{}, http.StatusForbidden, nil, nil)
+	testHTTP("GET", "/export", httpHeaders{hdrSessionBad}, http.StatusForbidden, nil, nil)
+	testHTTP("GET", "/export", httpHeaders{hdrSessionOK}, http.StatusForbidden, nil, nil)
+
+	testExport := func(format, archive, mailbox string, recursive bool, expectFiles int) {
+		t.Helper()
+
+		fields := url.Values{
+			"csrf":    []string{string(csrfToken)},
+			"format":  []string{format},
+			"archive": []string{archive},
+			"mailbox": []string{mailbox},
+		}
+		if recursive {
+			fields.Add("recursive", "on")
+		}
+		r := httptest.NewRequest("POST", "/export", strings.NewReader(fields.Encode()))
+		r.Header.Set("Content-Type", "application/x-www-form-urlencoded")
+		r.Header.Add("Cookie", cookieOK.String())
+		w := httptest.NewRecorder()
+		handle(apiHandler, false, "", w, r)
+		if w.Code != http.StatusOK {
+			t.Fatalf("export, got status code %d, expected 200: %s", w.Code, w.Body.Bytes())
+		}
+		var count int
+		if archive == "zip" {
+			buf := w.Body.Bytes()
+			zr, err := zip.NewReader(bytes.NewReader(buf), int64(len(buf)))
+			tcheck(t, err, "reading zip")
+			for _, f := range zr.File {
+				if !strings.HasSuffix(f.Name, "/") {
+					count++
+				}
+			}
+		} else {
+			var src io.Reader = w.Body
+			if archive == "tgz" {
+				gzr, err := gzip.NewReader(src)
+				tcheck(t, err, "gzip reader")
+				src = gzr
+			}
+			tr := tar.NewReader(src)
+			for {
+				h, err := tr.Next()
+				if err == io.EOF {
+					break
+				}
+				tcheck(t, err, "next file in tar")
+				if !strings.HasSuffix(h.Name, "/") {
+					count++
+				}
+				_, err = io.Copy(io.Discard, tr)
+				tcheck(t, err, "reading from tar")
+			}
+		}
+		if count != expectFiles {
+			t.Fatalf("export, has %d files, expected %d", count, expectFiles)
+		}
+	}
+
+	testExport("maildir", "tgz", "", true, 8+1) // 8 messages, 1 flags file
+	testExport("maildir", "zip", "", true, 8+1)
+	testExport("mbox", "tar", "", true, 6+5) // 6 default mailboxes, 5 created
+	testExport("mbox", "zip", "", true, 6+5)
+	testExport("mbox", "zip", "Lists", true, 3)
+	testExport("mbox", "zip", "Lists", false, 1)
+
 	// HTTP message, generic
 	testHTTP("GET", fmt.Sprintf("/msg/%v/attachments.zip", inboxMinimal.ID), nil, http.StatusForbidden, nil, nil)
 	testHTTP("GET", fmt.Sprintf("/msg/%v/attachments.zip", inboxMinimal.ID), httpHeaders{hdrCSRFBad}, http.StatusForbidden, nil, nil)
@@ -529,6 +613,11 @@ func TestWebmail(t *testing.T) {
 		"Content-Security-Policy",
 		"frame-ancestors 'self'; default-src 'none'; img-src data:; style-src 'unsafe-inline'; script-src 'unsafe-inline' 'self'; frame-src 'self'; connect-src 'self'",
 	}
+	// Text and img-src 'self', for viewing image files inline.
+	cspTextImg := [2]string{
+		"Content-Security-Policy",
+		"frame-ancestors 'self'; default-src 'none'; img-src data: 'self'; style-src 'unsafe-inline'; script-src 'unsafe-inline' 'self'; frame-src 'self'; connect-src 'self'",
+	}
 	// HTML as viewed in the regular viewer, not in a new tab.
 	cspHTML := [2]string{
 		"Content-Security-Policy",
@@ -559,7 +648,7 @@ func TestWebmail(t *testing.T) {
 		"Content-Security-Policy",
 		"frame-ancestors 'self'; default-src 'none'; img-src data: http: https: 'unsafe-inline'; style-src 'unsafe-inline' data: http: https:; font-src data: http: https: 'unsafe-inline'; media-src 'unsafe-inline' data: http: https:; script-src 'unsafe-inline' 'self'; frame-src 'self'; connect-src 'self'",
 	}
-	testHTTPAuthREST("GET", pathInboxAltRel+"/text", http.StatusOK, httpHeaders{ctHTML, cspText}, nil)
+	testHTTPAuthREST("GET", pathInboxAltRel+"/text", http.StatusOK, httpHeaders{ctHTML, cspTextImg}, nil)
 	testHTTPAuthREST("GET", pathInboxAltRel+"/html", http.StatusOK, httpHeaders{ctHTML, cspHTML}, nil)
 	testHTTPAuthREST("GET", pathInboxAltRel+"/htmlexternal", http.StatusOK, httpHeaders{ctHTML, cspHTMLExternal}, nil)
 	testHTTPAuthREST("GET", pathInboxAltRel+"/msgtext", http.StatusOK, httpHeaders{ctHTML, cspText}, nil)
@@ -595,7 +684,7 @@ func TestWebmail(t *testing.T) {
 	// Normally the generic /api/ auth check returns a user error. We bypass it and
 	// check for the server error.
 	sessionToken := store.SessionToken(strings.SplitN(sessionCookie.Value, " ", 2)[0])
-	reqInfo = requestInfo{"mjl@mox.example", "mjl", sessionToken, httptest.NewRecorder(), &http.Request{RemoteAddr: "127.0.0.1:1234"}}
+	reqInfo = requestInfo{log, "mjl@mox.example", acc, sessionToken, httptest.NewRecorder(), &http.Request{RemoteAddr: "127.0.0.1:1234"}}
 	ctx = context.WithValue(ctxbg, requestInfoCtxKey, reqInfo)
 	api.Logout(ctx)
 	tneedErrorCode(t, "server:error", func() { api.Logout(ctx) })
